//+------------------------------------------------------------------+
//|                                                         MaRS.mq4 |
//|                        Copyright 2019, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict

#property indicator_separate_window
#property indicator_minimum    10
#property indicator_maximum    100
#property indicator_level1     45.0
#property indicator_level2     55.0
#property indicator_level3     65.0

#property indicator_buffers 7
#property indicator_color1 clrSienna
#property indicator_color2 clrAliceBlue
#property indicator_color3 clrGold
#property indicator_color4 clrGold
#property indicator_color5 clrGreen
#property indicator_color6 clrRed

#define LAST 1

//--- indicator parameters
// MA line 1 (Fast)
extern int InpMAPeriod1=5; // Period
extern ENUM_TIMEFRAMES InpTIMEFRAME1=PERIOD_H1; //FastMA Anchor Timeframe
extern int InpMAShift1=0; // Shift
input ENUM_MA_METHOD InpMAMethod1=MODE_EMA; // Method

// MA line 2 (Slow)
extern int InpMAPeriod2=21; // Period
extern ENUM_TIMEFRAMES InpTIMEFRAME2=PERIOD_H1; //FastMA Anchor Timeframe
extern int InpMAShift2=0; // Shift
input ENUM_MA_METHOD InpMAMethod2=MODE_EMA; // Method

// inputs for MaRS
extern int InpMAPeriod_RS=30; // Period look back for MaRS
extern int MaRS_Avg_period=30; // Period computing MaRS average

int adjustedMA1, adjustedMA2;

//--- indicator buffer
double MaFastBuffer[], MaSlowBuffer[], MADiff_array[], MaRSBuffer[], MaRS_Avg[];
double TrendUp[], TrendDown[], State[]; //array for trendup/down
double MarketTypeBuffer[]; //array for enumerate MarketType info
double MyMAMultiplier;

//enum vars
enum ENUM_MARKETTYPE{DOWNTREND_OVERSOLD=1, 
					DOWNTREND_NORMAL=2,
					SIDEWAY=3,
					SIDEWAY_UP=4,
					SIDEWAY_DOWN=5,
					UPTREND_NORMAL=6,
					UPTREND_OVERBOUGHT=7
					};
					
enum ENUM_MaRS_VAL {_BTW45AND55=0, 
				   _LEQ45=1, 
				   _EQUAL55=2,
				   _BTW55AND65=3,
				   _GEQ65=4
					};
ENUM_MaRS_VAL MaRSvalEnum;
ENUM_MARKETTYPE MarketTypeEnum; //declare output var marketTypeEnum
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   ArraySetAsSeries(MaFastBuffer,true);
ArraySetAsSeries(MaSlowBuffer,true);
ArraySetAsSeries(MaRSBuffer,true);

int period = Period() ; //capture the timeframe of the chart use InpTIMEFRAME
MaAdjuster(InpTIMEFRAME1, period); //call the MaAdjuster function, sending the number of period() to the function
MaAdjuster(InpTIMEFRAME2, period); //call the MaAdjuster function, sending the number of period() to the function

adjustedMA1 = NormalizeDouble(InpMAPeriod1*MyMAMultiplier,0); //convert the number of candle base on selected TIMEFRAMES
adjustedMA2 = NormalizeDouble(InpMAPeriod2*MyMAMultiplier,0); //convert the number of candle base on selected TIMEFRAMES

int InpTIMEFRAME_RS = MathMax(InpTIMEFRAME1,InpTIMEFRAME2);

//--- indicator short name
  IndiShortName(InpMAMethod1);
  IndiShortName(InpMAMethod2);

//--- check for input
if(InpMAPeriod1<2 || InpMAPeriod2<2)
return(INIT_FAILED);
//--- indicator buffers mapping and drawings

SetIndexBuffer(0,MaFastBuffer); SetIndexStyle(0,DRAW_NONE); SetIndexShift(0,InpMAShift1); 
SetIndexLabel(0,"MaRS:MaFast"+tf2txt(InpTIMEFRAME1));

SetIndexBuffer(1,MaSlowBuffer);
SetIndexStyle(1,DRAW_NONE);
SetIndexShift(1,InpMAShift2);
SetIndexLabel(1,"MaRS:MaSlow"+tf2txt(InpTIMEFRAME2));

SetIndexBuffer(2,MaRSBuffer);
SetIndexStyle(2,DRAW_LINE,STYLE_SOLID,2);
SetIndexLabel(2,"MaRS"+tf2txt(InpTIMEFRAME_RS));

SetIndexBuffer(3,MaRS_Avg);
SetIndexStyle(3,DRAW_LINE,STYLE_SOLID,1);
SetIndexLabel(3,"MaRS:Avg"+tf2txt(InpTIMEFRAME_RS));

SetIndexBuffer(4,TrendUp); SetIndexStyle(4, DRAW_ARROW); SetIndexArrow(4, 236); 
SetIndexLabel(4,"MaRS:TrendUp");

SetIndexBuffer(5,TrendDown); SetIndexStyle(5, DRAW_ARROW); SetIndexArrow(5, 238);
SetIndexLabel(5,"MaRS:TrendDown");

SetIndexBuffer(6,MarketTypeBuffer); SetIndexStyle(6, DRAW_NONE);
SetIndexLabel(6,"MaRS:MarketType");
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int lookback = MathMax(InpMAMethod1,InpMAMethod2);
   int limit =Bars-1-MathMax(lookback, prev_calculated);
         
   int InpTIMEFRAME_RS = MathMax(InpTIMEFRAME1,InpTIMEFRAME2);
   //if(counted_bars==0) limit-=InpMAPeriod_RS;
   
   ArrayResize(TrendUp,limit);
   ArrayResize(TrendDown,limit);
   ArrayResize(State,limit);

//+-------main calculation---------------
//Computing the 2 MAs fast and slow and MaRS
for(int i=limit-1; i>=0; --i)
{
   MaFastBuffer[i] = iMA(NULL,0,adjustedMA1, InpMAShift1,InpMAMethod1,PRICE_CLOSE,i);
   MaSlowBuffer[i] = iMA(NULL,0,adjustedMA2, InpMAShift2,InpMAMethod2,PRICE_CLOSE,i);

   
   //Print("limit:", limit, "MARS_Period", InpMAPeriod_RS);

   //Computing the difference between 2 MA
      ArrayResize(MADiff_array,limit);
      MADiff_array[i] = NormalizeDouble(MaFastBuffer[i] - MaSlowBuffer[i],4);

   //Compute the MaRS buffer 
   MaRSBuffer[i] = iRSIOnArray(MADiff_array,0,InpMAPeriod_RS,i);       
  }
  
  //calculate the last x value of MaRSBuffer array
  //ArraySetAsSeries(MaRS_Avg,true);
  for(int i=limit-2; i>=0; --i)
   {
   //initial value
    //string logmsg = "i: " + i + "Size of Array Trend:" + ArraySize(Trend) + " " + iMAOnArray(MaRSBuffer,0,MaRS_Avg_period,0,MODE_SMA,i) ;
    //  Print(logmsg);
      MaRS_Avg[i] = iMAOnArray(MaRSBuffer,0,MaRS_Avg_period,0,MODE_SMA,i);
      TrendUp[i]   = 0;
      TrendDown[i] = 0;
      //double range  = iATR(NULL,0,10,i);
      State[i]     = State[i+1];
      if (MaRSBuffer[i]>MaRS_Avg[i]) State[i] =  1; //trend up
      if (MaRSBuffer[i]<MaRS_Avg[i]) State[i] = -1; // trend down
    
    // if current bar state is different from previous bar
      if (State[i] != State[i+1]) 
       if (State[i] == 1)
               TrendUp[i]   = MaRSBuffer[i] - 12.0;
         else  TrendDown[i] = MaRSBuffer[i]+ 12.0;
    }
   
      // get the value defining market Type:

    for(int i=limit-1; i>=0; --i)
      {
      MarketTypeBuffer[i] = MarketTypeConvert(MaRSBuffer,TrendUp,TrendDown,i) ; //assign the value the the array
      }
   
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+

int MarketTypeConvert(const double& IndiArray[], const double& UpArray[], const double& DownArray[], int i )
{
   double IndiVal, TrendVal;
   
   //defining market type looping thru input array IndiArray
    IndiVal = IndiArray[i]; 
    //get the value for TrendVal
    if(UpArray[i] >0)TrendVal=1;
      else if (DownArray[i] > 0)TrendVal=-1; 
         else TrendVal = 0;
    
    //get the value for MaRSvalEnum
    if(IndiArray[i]<=54 && IndiArray[i]>45)MaRSvalEnum=_BTW45AND55;
      else if(IndiArray[i]<=45)MaRSvalEnum=_LEQ45;
      else if(IndiArray[i]>54 && IndiArray[i]<56)MaRSvalEnum=_EQUAL55;
      else if(IndiArray[i]>=56 && IndiArray[i]<65)MaRSvalEnum=_BTW55AND65;
      else if(IndiArray[i]>=65)MaRSvalEnum=_GEQ65;
      
     //get value for MarketTypeEnum
     switch(MaRSvalEnum) //convert to daily MA
      {
         case _LEQ45: MarketTypeEnum =DOWNTREND_OVERSOLD; break;
         case _BTW45AND55: if(TrendVal<=0){MarketTypeEnum =DOWNTREND_NORMAL;} else{MarketTypeEnum=SIDEWAY_UP;} break;
         case _BTW55AND65: if(TrendVal>=0){MarketTypeEnum =UPTREND_NORMAL;}else{MarketTypeEnum=SIDEWAY_DOWN;}break;
         case _GEQ65: MarketTypeEnum =UPTREND_OVERBOUGHT; break;
         default:MarketTypeEnum=SIDEWAY;
      }
    return(MarketTypeEnum);
}


//---------------------------------------  
double ArrayAverage(double& array[])
   {
      double summation = 0; 
      for (int iArray = ArraySize(array) - 1; iArray >= 0; iArray--) summation += array[iArray];
      double average =    summation / ArraySize(array);
      if(average!=0)return(average);
      else 
         {
         Print("Error !!!", GetLastError());
         return(0);
         }
   }
   
 //+------------------------------------------------------------------+

   void MaAdjuster(int anchorPeriod, int number)
   {
      if(anchorPeriod==PERIOD_M30)
      {
         switch(number) //convert to daily MA
         {
         case PERIOD_M1:MyMAMultiplier=30;break;
         case PERIOD_M5:MyMAMultiplier=6;break;
         case PERIOD_M15:MyMAMultiplier=2;break;
         case PERIOD_M30:MyMAMultiplier=1;break;
         case PERIOD_H1:MyMAMultiplier=0.5;break;
         case PERIOD_H4:MyMAMultiplier=0.125;break;
         case PERIOD_D1:MyMAMultiplier=0.0208;break;
         }
      }
      
      if(anchorPeriod==PERIOD_H1)
      {
         switch(number) //convert to daily MA
         {
         case PERIOD_M1:MyMAMultiplier=60;break;
         case PERIOD_M5:MyMAMultiplier=12;break;
         case PERIOD_M15:MyMAMultiplier=4;break;
         case PERIOD_M30:MyMAMultiplier=2;break;
         case PERIOD_H1:MyMAMultiplier=1;break;
         case PERIOD_H4:MyMAMultiplier=0.25;break;
         case PERIOD_D1:MyMAMultiplier=0.042;break;
         }
      }
      if(anchorPeriod==PERIOD_H4)
      {
         switch(number) //convert to daily MA
         {
         case PERIOD_M1:MyMAMultiplier=240;break;
         case PERIOD_M5:MyMAMultiplier=48;break;
         case PERIOD_M15:MyMAMultiplier=16;break;
         case PERIOD_M30:MyMAMultiplier=8;break;
         case PERIOD_H1:MyMAMultiplier=4;break;
         case PERIOD_H4:MyMAMultiplier=1;break;
         case PERIOD_D1:MyMAMultiplier=0.1667;break;
         }
      }
      if(anchorPeriod==PERIOD_D1)
      {
         switch(number) //convert to daily MA
         {
         case PERIOD_M1:MyMAMultiplier=3600;break;
         case PERIOD_M5:MyMAMultiplier=288;break;
         case PERIOD_M15:MyMAMultiplier=96;break;
         case PERIOD_M30:MyMAMultiplier=48;break;
         case PERIOD_H1:MyMAMultiplier=24;break;
         case PERIOD_H4:MyMAMultiplier=6;break;
         case PERIOD_D1:MyMAMultiplier=1;break;
         }
      }
   }

   string tf2txt(int tf)
   {
      if(tf==PERIOD_M1) return("M1");
      if(tf==PERIOD_M5) return("M5");
      if(tf==PERIOD_M15) return("M15");
      if(tf==PERIOD_M30) return("M30");
      if(tf==PERIOD_H1) return("H1");
      if(tf==PERIOD_H4) return("H4");
      if(tf==PERIOD_D1) return("D1");
      if(tf==PERIOD_W1) return("W1");
      if(tf==PERIOD_MN1) return("MN1");
      //----
      return("??");
   }
//+------------------------------------------------------------------+

string MarketType2txt(ENUM_MARKETTYPE MT)
{
  if(MT==DOWNTREND_NORMAL) return("DownNormal");
      if(MT==DOWNTREND_OVERSOLD) return("Oversold");
      if(MT==UPTREND_NORMAL) return("UpNormal");
      if(MT==UPTREND_OVERBOUGHT) return("UpOverBought");
      if(MT==SIDEWAY) return("Sideway");
      if(MT==SIDEWAY_UP) return("SidewayUp");
      if(MT==SIDEWAY_DOWN) return("SidewayDown");
   return("??");
}

//--- indicator short name
int IndiShortName(int maMethod)
{
   string short_name;
   switch(maMethod)
   {
   case MODE_SMA : short_name="SMA("; break;
   case MODE_EMA : short_name="EMA(";  break;
   case MODE_SMMA : short_name="SMMA("; break;
   case MODE_LWMA : short_name="LWMA("; break;
   default : return(INIT_FAILED);
   }
   IndicatorShortName(short_name+string(maMethod)+")");
   IndicatorDigits(Digits);
   return(0);
}